==> javie-0.3.9-src.tar.gz: javie-0.3.9-src/ch.kuramo.javie.app/src/main/java/ch/kuramo/javie/app/project/NewLayerCompositionOperation.java <==
/*
 * Copyright (c) 2009 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.app.project;

import java.util.Collections;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import ch.kuramo.javie.api.ColorMode;
import ch.kuramo.javie.api.Size2i;
import ch.kuramo.javie.api.Time;
import ch.kuramo.javie.app.InjectorHolder;
import ch.kuramo.javie.core.Composition;
import ch.kuramo.javie.core.CompositionItem;
import ch.kuramo.javie.core.Folder;
import ch.kuramo.javie.core.Item;
import ch.kuramo.javie.core.LayerComposition;
import ch.kuramo.javie.core.Project;
import ch.kuramo.javie.core.ProjectDecodeException;
import ch.kuramo.javie.core.services.ProjectDecoder;
import ch.kuramo.javie.core.services.ProjectElementFactory;
import ch.kuramo.javie.core.services.ProjectEncoder;

import com.google.inject.Inject;

public class NewLayerCompositionOperation extends ProjectOperation {

	private final String _compId;

	private final String _itemId;

	private final String _data;

	@Inject
	private ProjectElementFactory _elementFactory;

	@Inject
	private ProjectEncoder _encoder;

	@Inject
	private ProjectDecoder _decoder;


	public NewLayerCompositionOperation(ProjectManager projectManager,
			Folder parent, String name, int width, int height,
			Time frameDuration, Time duration, ColorMode colorMode) {
		super(projectManager, "新規コンポジション");
		projectManager.checkItem(parent);

		InjectorHolder.getInjector().injectMembers(this);

		LayerComposition comp = _elementFactory.newLayerComposition(
				colorMode, new Size2i(width, height), frameDuration, duration);

		CompositionItem item = _elementFactory.newCompositionItem(comp);
		item.setName(name);
		item.setParent(parent);

		_compId = comp.getId();
		_itemId = item.getId();
		_data = _encoder.encodeElement(new Object[] { comp, item });
	}

	@Override
	protected IStatus executeOrRedo(IProgressMonitor monitor, IAdaptable info,
			Project project, ProjectManager pm) throws ExecutionException {

		LayerComposition comp = null;
		CompositionItem item = null;

		try {
			Object[] elements = _decoder.decodeElement(_data, Object[].class);
			comp = (LayerComposition) elements[0];
			item = (CompositionItem) elements[1];

			project.getCompositions().add(comp);
			project.getItems().add(item);

			comp.afterDecode(project);
			item.afterDecode(project);

		} catch (ProjectDecodeException e) {
			cleanup(project, comp, item);
			throw new ExecutionException("error decoding SolidColorItem data", e);
		} catch (RuntimeException e) {
			cleanup(project, comp, item);
			throw e;
		}

		if (pm != null) {
			fireItemsAdd(item, pm);
		}

		return Status.OK_STATUS;
	}

	@Override
	protected IStatus undo(IProgressMonitor monitor, IAdaptable info,
			Project project, ProjectManager pm) throws ExecutionException {

		Item item = project.getItem(_itemId);
		Composition comp = project.getComposition(_compId);

		project.getItems().remove(item);
		project.getCompositions().remove(comp);

		if (pm != null) {
			fireItemsRemove(item, pm);
		}

		item.dispose();
		return Status.OK_STATUS;
	}

	private void fireItemsAdd(Item item, ProjectManager pm) {
		Folder parent = item.getParent();
		if (parent != null) {
			pm.fireItemsAdd(Collections.singleton(item), Collections.singleton(parent));
		} else {
			pm.fireItemsAdd(Collections.singleton(item), null);
		}
	}

	private void fireItemsRemove(Item item, ProjectManager pm) {
		Folder parent = item.getParent();
		if (parent != null) {
			pm.fireItemsRemove(Collections.singleton(item), Collections.singleton(parent));
		} else {
			pm.fireItemsRemove(Collections.singleton(item), null);
		}
	}

	private void cleanup(Project project, Composition comp, Item item) {
		project.getCompositions().remove(comp);
		project.getItems().remove(item);
		if (item != null) {
			item.dispose();
		}
	}

	public String getCompositionItemId() {
		return _itemId;
	}

}

==> javie-0.3.12-src.tar.gz: javie-0.3.12-src/ch.kuramo.javie.app/src/main/java/ch/kuramo/javie/app/project/NewLayerCompositionOperation.java <==
/*
 * Copyright (c) 2009,2010 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.app.project;

import java.util.Collections;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import ch.kuramo.javie.api.ColorMode;
import ch.kuramo.javie.api.Size2i;
import ch.kuramo.javie.api.Time;
import ch.kuramo.javie.app.InjectorHolder;
import ch.kuramo.javie.core.Composition;
import ch.kuramo.javie.core.CompositionItem;
import ch.kuramo.javie.core.Folder;
import ch.kuramo.javie.core.Item;
import ch.kuramo.javie.core.LayerComposition;
import ch.kuramo.javie.core.Project;
import ch.kuramo.javie.core.ProjectDecodeException;
import ch.kuramo.javie.core.services.ProjectDecoder;
import ch.kuramo.javie.core.services.ProjectElementFactory;
import ch.kuramo.javie.core.services.ProjectEncoder;

import com.google.inject.Inject;

public class NewLayerCompositionOperation extends ProjectOperation {

	private final String _compId;

	private final String _itemId;

	private final String _data;

	@Inject
	private ProjectElementFactory _elementFactory;

	@Inject
	private ProjectEncoder _encoder;

	@Inject
	private ProjectDecoder _decoder;


	public NewLayerCompositionOperation(ProjectManager projectManager,
			Folder parent, String name, ColorMode colorMode, Size2i size,
			Time frameDuration, Time duration, boolean frameDurationPreserved,
			double motionBlurShutterAngle, double motionBlurShutterPhase, int motionBlurSamples) {

		super(projectManager, "新規コンポジション", true);
		projectManager.checkItem(parent);

		InjectorHolder.getInjector().injectMembers(this);

		LayerComposition comp = _elementFactory.newLayerComposition(colorMode, size, frameDuration, duration);
		comp.setFrameDurationPreserved(frameDurationPreserved);
		comp.setMotionBlurShutterAngle(motionBlurShutterAngle);
		comp.setMotionBlurShutterPhase(motionBlurShutterPhase);
		comp.setMotionBlurSamples(motionBlurSamples);

		CompositionItem item = _elementFactory.newCompositionItem(comp);
		item.setName(name);
		item.setParent(parent);

		_compId = comp.getId();
		_itemId = item.getId();
		_data = _encoder.encodeElement(new Object[] { comp, item });
	}

	@Override
	protected IStatus executeOrRedo(IProgressMonitor monitor, IAdaptable info,
			Project project, ProjectManager pm) throws ExecutionException {

		LayerComposition comp = null;
		CompositionItem item = null;

		try {
			Object[] elements = _decoder.decodeElement(_data, Object[].class);
			comp = (LayerComposition) elements[0];
			item = (CompositionItem) elements[1];

			project.getCompositions().add(comp);
			project.getItems().add(item);

			comp.afterDecode(project);
			item.afterDecode(project);

		} catch (ProjectDecodeException e) {
			cleanup(project, comp, item);
			throw new ExecutionException("error decoding SolidColorItem data", e);
		} catch (RuntimeException e) {
			cleanup(project, comp, item);
			throw e;
		}

		if (pm != null) {
			fireItemsAdd(item, pm);
		}

		return Status.OK_STATUS;
	}

	@Override
	protected IStatus undo(IProgressMonitor monitor, IAdaptable info,
			Project project, ProjectManager pm) throws ExecutionException {

		Item item = project.getItem(_itemId);
		Composition comp = project.getComposition(_compId);

		project.getItems().remove(item);
		project.getCompositions().remove(comp);

		if (pm != null) {
			fireItemsRemove(item, pm);
		}

		item.dispose();
		return Status.OK_STATUS;
	}

	private void fireItemsAdd(Item item, ProjectManager pm) {
		Folder parent = item.getParent();
		if (parent != null) {
			pm.fireItemsAdd(Collections.singleton(item), Collections.singleton(parent));
		} else {
			pm.fireItemsAdd(Collections.singleton(item), null);
		}
	}

	private void fireItemsRemove(Item item, ProjectManager pm) {
		Folder parent = item.getParent();
		if (parent != null) {
			pm.fireItemsRemove(Collections.singleton(item), Collections.singleton(parent));
		} else {
			pm.fireItemsRemove(Collections.singleton(item), null);
		}
	}

	private void cleanup(Project project, Composition comp, Item item) {
		project.getCompositions().remove(comp);
		project.getItems().remove(item);
		if (item != null) {
			item.dispose();
		}
	}

	public String getCompositionItemId() {
		return _itemId;
	}

}
